#include <iostream>

#include "solver.hpp"

namespace solver {

std::optional<solver::TimeSettings> make_time(solver::InputInfo ii) {
  solver::TimeSettings time;

  auto dt_max = std::get<solver::NumberArray>(ii[DT_MAX]);
  if (dt_max.size() != 1) {
    std::cout << "dt_max is a scalar: " << dt_max.size() << std::endl;
  } else {
    time.dt_max = dt_max[0];
  }

  auto dt_min = std::get<solver::NumberArray>(ii[DT_MIN]);
  if (dt_min.size() != 1) {
    std::cout << "dt_min is a scalar" << std::endl;
  } else {
    time.dt_min = dt_min[0];
  }

  auto maxstep = std::get<solver::NumberArray>(ii[MAXSTEP]);
  if (maxstep.size() != 1) {
    std::cout << "max_step is a scalar" << std::endl;
  } else {
    time.max_step = maxstep[0];
  }

  if (!ii.count(TSTOP)) {
    require(TSTOP);
    return std::nullopt;
  }
  auto tstop = std::get<solver::NumberArray>(ii[TSTOP]);
  if (tstop.size() != 1) {
    std::cout << "tstop is a scalar" << std::endl;
  } else {
    time.tstop = tstop[0];
  }

  auto fixed_dt = std::get<solver::NumberArray>(ii[FIXED_DT]);
  if (fixed_dt.size() != 1) {
    std::cout << "fixed_dt is a scalar" << std::endl;
  } else {
    time.fixed_dt = (fixed_dt[0] != 0);
  }

  auto cfl = std::get<solver::NumberArray>(ii[CFL]);
  if (cfl.size() != 1) {
    std::cout << "cfl is a scalar" << std::endl;
  } else {
    time.cfl = cfl[0];
  }

  auto tcoll_ratio = std::get<solver::NumberArray>(ii[TCOLL_RATIO]);
  if (tcoll_ratio.size() != 1) {
    std::cout << "tcoll_ratio is a scalar" << std::endl;
  } else {
    time.tcoll_ratio = tcoll_ratio[0];
  }

  return time;
}
} // namespace solver
